@@ -38,10 +38,15 @@
  */
 #undef DEINTERLACE_ON_NOT_NORMAL_SPEED
 
+/** This is not the ideal place for this, but it is needed by VDPAU as well.
+ */
+pthread_mutex_t mlt_sdl_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 static void mlt_consumer_frame_render( mlt_listener listener, mlt_properties owner, mlt_service this, void **args );
 static void mlt_consumer_frame_show( mlt_listener listener, mlt_properties owner, mlt_service this, void **args );
-static void mlt_consumer_property_changed( mlt_service owner, mlt_consumer this, char *name );
+static void mlt_consumer_property_changed( mlt_properties owner, mlt_consumer this, char *name );
 static void apply_profile_properties( mlt_consumer this, mlt_profile profile, mlt_properties properties );
+static void on_consumer_frame_show( mlt_properties owner, mlt_consumer this, mlt_frame frame );
 
 /** Initialize a consumer service.
  *
@@ -97,6 +102,7 @@ int mlt_consumer_init( mlt_consumer this, void *child, mlt_profile profile )
 		mlt_events_register( properties, "consumer-frame-show", ( mlt_transmitter )mlt_consumer_frame_show );
 		mlt_events_register( properties, "consumer-frame-render", ( mlt_transmitter )mlt_consumer_frame_render );
 		mlt_events_register( properties, "consumer-stopped", NULL );
+		mlt_events_listen( properties, this, "consumer-frame-show", ( mlt_listener )on_consumer_frame_show );
 
 		// Register a property-changed listener to handle the profile property -
 		// subsequent properties can override the profile
@@ -133,18 +139,19 @@ static void apply_profile_properties( mlt_consumer this, mlt_profile profile, ml
 	mlt_properties_set_double( properties, "display_ratio", mlt_profile_dar( profile )  );
 	mlt_properties_set_int( properties, "display_aspect_num", profile->display_aspect_num );
 	mlt_properties_set_int( properties, "display_aspect_num", profile->display_aspect_num );
+	mlt_properties_set_int( properties, "colorspace", profile->colorspace );
 	mlt_event_unblock( this->event_listener );
 }
 
 /** The property-changed event listener
  *
  * \private \memberof mlt_consumer_s
- * \param owner the service a service (ignored)
+ * \param owner the events object
  * \param this the consumer
  * \param name the name of the property that changed
  */
 
-static void mlt_consumer_property_changed( mlt_service owner, mlt_consumer this, char *name )
+static void mlt_consumer_property_changed( mlt_properties owner, mlt_consumer this, char *name )
 {
 	if ( !strcmp( name, "profile" ) )
 	{
@@ -253,6 +260,13 @@ static void mlt_consumer_property_changed( mlt_service owner, mlt_consumer this,
 			mlt_properties_set_double( properties, "display_ratio", mlt_profile_dar( profile )  );
 		}
 	}
+	else if ( !strcmp( name, "colorspace" ) )
+	{
+		mlt_properties properties = MLT_CONSUMER_PROPERTIES( this );
+		mlt_profile profile = mlt_service_profile( MLT_CONSUMER_SERVICE( this ) );
+		if ( profile )
+			profile->colorspace = mlt_properties_get_int( properties, "colorspace" );
+	}
 }
 
 /** The transmitter for the consumer-frame-show event
@@ -261,7 +275,7 @@ static void mlt_consumer_property_changed( mlt_service owner, mlt_consumer this,
  *
  * \private \memberof mlt_consumer_s
  * \param listener a function pointer that will be invoked
- * \param owner  a properties list that will be passed to \p listener
+ * \param owner the events object that will be passed to \p listener
  * \param this  a service that will be passed to \p listener
  * \param args an array of pointers - the first entry is passed as a string to \p listener
  */
@@ -278,7 +292,7 @@ static void mlt_consumer_frame_show( mlt_listener listener, mlt_properties owner
  *
  * \private \memberof mlt_consumer_s
  * \param listener a function pointer that will be invoked
- * \param owner  a properties list that will be passed to \p listener
+ * \param owner the events object that will be passed to \p listener
  * \param this  a service that will be passed to \p listener
  * \param args an array of pointers - the first entry is passed as a string to \p listener
  */
@@ -289,6 +303,22 @@ static void mlt_consumer_frame_render( mlt_listener listener, mlt_properties own
 		listener( owner, this, ( mlt_frame )args[ 0 ] );
 }
 
+/** A listener on the consumer-frame-show event
+ *
+ * Saves the position of the frame shown.
+ *
+ * \private \memberof mlt_consumer_s
+ * \param owner the events object
+ * \param consumer the consumer on which this event occurred
+ * \param frame the frame that was shown
+ */
+
+static void on_consumer_frame_show( mlt_properties owner, mlt_consumer consumer, mlt_frame frame )
+{
+	if ( frame )
+		consumer->position = mlt_frame_get_position( frame );
+}
+
 /** Create a new consumer.
  *
  * \public \memberof mlt_consumer_s
@@ -1008,3 +1038,16 @@ void mlt_consumer_close( mlt_consumer this )
 		}
 	}
 }
+
+/** Get the position of the last frame shown.
+ *
+ * \public \memberof mlt_consumer_s
+ * \param consumer a consumer
+ * \return the position
+ */
+
+mlt_position mlt_consumer_position( mlt_consumer consumer )
+{
+	return consumer->position;
+}
+		
