@@ -57,6 +57,8 @@ typedef int ( *mlt_get_audio )( mlt_frame self, void **buffer, mlt_audio_format
  * (no speed factor applied, only available when \em _need_previous_next is set on the producer)
  * \properties \em next \em frame a reference to the unfiltered following frame
  * (no speed factor applied, only available when \em _need_previous_next is set on the producer)
+ * \properties \em colorspace the standard for luma coefficients
+ * \properties \em force_full_luma luma range handling, set to -1 for pass-through, 1 for full range, 0 for scaling
  */
 
 struct mlt_frame_s
@@ -130,6 +132,7 @@ extern int mlt_sample_calculator( float fps, int frequency, int64_t position );
 extern int64_t mlt_sample_calculator_to_now( float fps, int frequency, int64_t position );
 extern const char * mlt_image_format_name( mlt_image_format format );
 extern const char * mlt_audio_format_name( mlt_audio_format format );
+extern void mlt_frame_write_ppm( mlt_frame frame );
 
 /** This macro scales RGB into the YUV gamut - y is scaled by 219/255 and uv by 224/255. */
 #define RGB2YUV_601_SCALED(r, g, b, y, u, v)\
