@@ -246,6 +246,15 @@ int mlt_property_set_data( mlt_property this, void *value, int length, mlt_destr
  *
  * The string must begin with '0x' to be interpreted as hexadecimal.
  * Otherwise, it is interpreted as base 10.
+ * If the string begins with '#' it is interpreted as a hexadecimal color value
+ * in the form RRGGBB or AARRGGBB. Color values that begin with '0x' are
+ * always in the form RRGGBBAA where the alpha components are not optional.
+ * Applications and services should expect the binary color value in bytes to
+ * be in the following order: RGBA. This means they will have to cast the int
+ * to an unsigned int. This is especially important when they need to shift
+ * right to obtain RGB without alpha in order to make it do a logical instead
+ * of arithmetic shift.
+ *
  * \private \memberof mlt_property_s
  * \param value a string to convert
  * \return the resultant integer
@@ -254,10 +263,23 @@ static inline int mlt_property_atoi( const char *value )
 {
 	if ( value == NULL )
 		return 0;
+	// Parse a hex color value as #RRGGBB or #AARRGGBB.
+	if ( value[0] == '#' )
+	{
+		unsigned int rgb = strtoul( value + 1, NULL, 16 );
+		unsigned int alpha = ( strlen( value ) > 7 ) ? ( rgb >> 24 ) : 0xff;
+		return ( rgb << 8 ) | alpha;
+	}
+	// Do hex and decimal explicitly to avoid decimal value with leading zeros
+	// interpreted as octal.
 	else if ( value[0] == '0' && value[1] == 'x' )
-		return strtol( value + 2, NULL, 16 );
+	{
+		return strtoul( value + 2, NULL, 16 );
+	}
 	else
+	{
 		return strtol( value, NULL, 10 );
+	}
 }
 
 /** Get the property as an integer.
@@ -277,7 +299,7 @@ int mlt_property_get_int( mlt_property this )
 		return ( int )this->prop_position;
 	else if ( this->types & mlt_prop_int64 )
 		return ( int )this->prop_int64;
-	else if ( this->types & mlt_prop_string )
+	else if ( ( this->types & mlt_prop_string ) && this->prop_string )
 		return mlt_property_atoi( this->prop_string );
 	return 0;
 }
@@ -299,7 +321,7 @@ double mlt_property_get_double( mlt_property this )
 		return ( double )this->prop_position;
 	else if ( this->types & mlt_prop_int64 )
 		return ( double )this->prop_int64;
-	else if ( this->types & mlt_prop_string )
+	else if ( ( this->types & mlt_prop_string ) && this->prop_string )
 		return atof( this->prop_string );
 	return 0;
 }
@@ -322,7 +344,7 @@ mlt_position mlt_property_get_position( mlt_property this )
 		return ( mlt_position )this->prop_double;
 	else if ( this->types & mlt_prop_int64 )
 		return ( mlt_position )this->prop_int64;
-	else if ( this->types & mlt_prop_string )
+	else if ( ( this->types & mlt_prop_string ) && this->prop_string )
 		return ( mlt_position )atol( this->prop_string );
 	return 0;
 }
@@ -362,7 +384,7 @@ int64_t mlt_property_get_int64( mlt_property this )
 		return ( int64_t )this->prop_double;
 	else if ( this->types & mlt_prop_position )
 		return ( int64_t )this->prop_position;
-	else if ( this->types & mlt_prop_string )
+	else if ( ( this->types & mlt_prop_string ) && this->prop_string )
 		return mlt_property_atoll( this->prop_string );
 	return 0;
 }
@@ -405,7 +427,7 @@ char *mlt_property_get_string( mlt_property this )
 		{
 			this->types |= mlt_prop_string;
 			this->prop_string = malloc( 32 );
-			sprintf( this->prop_string, "%lld", this->prop_int64 );
+                        sprintf( this->prop_string, "%lld", (long long int)this->prop_int64 );
 		}
 		else if ( this->types & mlt_prop_data && this->serialiser != NULL )
 		{
@@ -428,7 +450,7 @@ char *mlt_property_get_string( mlt_property this )
  * destructor function.
  * \public \memberof mlt_property_s
  * \param this a property
- * \param[out] length the size of the binary object in bytes
+ * \param[out] length the size of the binary object in bytes (optional)
  * \return an opaque data pointer or NULL if not available
  */
 
