@@ -24,6 +24,7 @@
 #include <framework/mlt_frame.h>
 #include <framework/mlt_profile.h>
 #include <framework/mlt_log.h>
+#include <framework/mlt_events.h>
 
 // System header files
 #include <stdio.h>
@@ -35,11 +36,11 @@
 #include <unistd.h>
 
 // avformat header files
-#include <avformat.h>
+#include <libavformat/avformat.h>
 #ifdef SWSCALE
-#include <swscale.h>
+#include <libswscale/swscale.h>
 #endif
-#include <opt.h>
+#include <libavcodec/opt.h>
 #if LIBAVUTIL_VERSION_INT >= ((50<<16)+(8<<8)+0)
 #include <libavutil/pixdesc.h>
 #endif
@@ -51,6 +52,11 @@
 
 #define MAX_AUDIO_STREAMS (8)
 #define AUDIO_ENCODE_BUFFER_SIZE (48000 * 2 * MAX_AUDIO_STREAMS)
+#define AUDIO_BUFFER_SIZE (1024 * 42)
+#define VIDEO_BUFFER_SIZE (2048 * 1024)
+
+void avformat_lock( );
+void avformat_unlock( );
 
 //
 // This structure should be extended and made globally available in mlt
@@ -192,6 +198,8 @@ mlt_consumer consumer_avformat_init( mlt_profile profile, char *arg )
 		this->start = consumer_start;
 		this->stop = consumer_stop;
 		this->is_stopped = consumer_is_stopped;
+		
+		mlt_events_register( properties, "consumer-fatal-error", NULL );
 	}
 
 	// Return this
@@ -211,33 +219,60 @@ static int consumer_start( mlt_consumer this )
 	char *s = mlt_properties_get( properties, "f" );
 	if ( s && strcmp( s, "list" ) == 0 )
 	{
-		fprintf( stderr, "---\nformats:\n" );
+		mlt_properties doc = mlt_properties_new();
+		mlt_properties formats = mlt_properties_new();
+		char key[20];
 		AVOutputFormat *format = NULL;
+		
+		mlt_properties_set_data( properties, "f", formats, 0, (mlt_destructor) mlt_properties_close, NULL );
+		mlt_properties_set_data( doc, "formats", formats, 0, NULL, NULL );
 		while ( ( format = av_oformat_next( format ) ) )
-			fprintf( stderr, "  - %s\n", format->name );
-		fprintf( stderr, "...\n" );
+		{
+			snprintf( key, sizeof(key), "%d", mlt_properties_count( formats ) );
+			mlt_properties_set( formats, key, format->name );
+		}
+		fprintf( stderr, "%s", mlt_properties_serialise_yaml( doc ) );
+		mlt_properties_close( doc );
 		error = 1;
 	}
 	s = mlt_properties_get( properties, "acodec" );
 	if ( s && strcmp( s, "list" ) == 0 )
 	{
-		fprintf( stderr, "---\naudio_codecs:\n" );
+		mlt_properties doc = mlt_properties_new();
+		mlt_properties codecs = mlt_properties_new();
+		char key[20];
 		AVCodec *codec = NULL;
+
+		mlt_properties_set_data( properties, "acodec", codecs, 0, (mlt_destructor) mlt_properties_close, NULL );
+		mlt_properties_set_data( doc, "audio_codecs", codecs, 0, NULL, NULL );
 		while ( ( codec = av_codec_next( codec ) ) )
 			if ( codec->encode && codec->type == CODEC_TYPE_AUDIO )
-				fprintf( stderr, "  - %s\n", codec->name );
-		fprintf( stderr, "...\n" );
+			{
+				snprintf( key, sizeof(key), "%d", mlt_properties_count( codecs ) );
+				mlt_properties_set( codecs, key, codec->name );
+			}
+		fprintf( stderr, "%s", mlt_properties_serialise_yaml( doc ) );
+		mlt_properties_close( doc );
 		error = 1;
 	}
 	s = mlt_properties_get( properties, "vcodec" );
 	if ( s && strcmp( s, "list" ) == 0 )
 	{
-		fprintf( stderr, "---\nvideo_codecs:\n" );
+		mlt_properties doc = mlt_properties_new();
+		mlt_properties codecs = mlt_properties_new();
+		char key[20];
 		AVCodec *codec = NULL;
+
+		mlt_properties_set_data( properties, "vcodec", codecs, 0, (mlt_destructor) mlt_properties_close, NULL );
+		mlt_properties_set_data( doc, "video_codecs", codecs, 0, NULL, NULL );
 		while ( ( codec = av_codec_next( codec ) ) )
 			if ( codec->encode && codec->type == CODEC_TYPE_VIDEO )
-				fprintf( stderr, "  - %s\n", codec->name );
-		fprintf( stderr, "...\n" );
+			{
+				snprintf( key, sizeof(key), "%d", mlt_properties_count( codecs ) );
+				mlt_properties_set( codecs, key, codec->name );
+			}
+		fprintf( stderr, "%s", mlt_properties_serialise_yaml( doc ) );
+		mlt_properties_close( doc );
 		error = 1;
 	}
 
@@ -463,6 +498,8 @@ static int open_audio( AVFormatContext *oc, AVStream *st, int audio_outbuf_size
 	// Find the encoder
 	AVCodec *codec = avcodec_find_encoder( c->codec_id );
 
+	avformat_lock();
+	
 	// Continue if codec found and we can open it
 	if ( codec != NULL && avcodec_open( c, codec ) >= 0 )
 	{
@@ -498,6 +535,8 @@ static int open_audio( AVFormatContext *oc, AVStream *st, int audio_outbuf_size
 	{
 		mlt_log_warning( NULL, "%s: Unable to encode audio - disabling audio output.\n", __FILE__ );
 	}
+	
+	avformat_unlock();
 
 	return audio_input_frame_size;
 }
@@ -505,7 +544,11 @@ static int open_audio( AVFormatContext *oc, AVStream *st, int audio_outbuf_size
 static void close_audio( AVFormatContext *oc, AVStream *st )
 {
 	if ( st && st->codec )
+	{
+		avformat_lock();
 		avcodec_close( st->codec );
+		avformat_unlock();
+	}
 }
 
 /** Add a video output stream 
@@ -542,10 +585,37 @@ static AVStream *add_video_stream( mlt_consumer this, AVFormatContext *oc, int c
 		if ( vpre )
 		{
 			mlt_properties p = mlt_properties_load( vpre );
+#ifdef AVDATADIR
+			if ( mlt_properties_count( p ) < 1 )
+			{
+				AVCodec *codec = avcodec_find_encoder( c->codec_id );
+				if ( codec )
+				{
+					char *path = malloc( strlen(AVDATADIR) + strlen(codec->name) + strlen(vpre) + strlen(".ffpreset") + 2 );
+					strcpy( path, AVDATADIR );
+					strcat( path, codec->name );
+					strcat( path, "-" );
+					strcat( path, vpre );
+					strcat( path, ".ffpreset" );
+					
+					mlt_properties_close( p );
+					p = mlt_properties_load( path );
+					mlt_properties_debug( p, path, stderr );
+					free( path );	
+				}
+			}
+			else
+			{
+				mlt_properties_debug( p, vpre, stderr );			
+			}
+#endif
 			apply_properties( c, p, AV_OPT_FLAG_VIDEO_PARAM | AV_OPT_FLAG_ENCODING_PARAM, 1 );
 			mlt_properties_close( p );
 		}
+		int colorspace = mlt_properties_get_int( properties, "colorspace" );
+		mlt_properties_set( properties, "colorspace", NULL );
 		apply_properties( c, properties, AV_OPT_FLAG_VIDEO_PARAM | AV_OPT_FLAG_ENCODING_PARAM, 0 );
+		mlt_properties_set_int( properties, "colorspace", colorspace );
 
 		// Set options controlled by MLT
 		c->width = mlt_properties_get_int( properties, "width" );
@@ -559,6 +629,27 @@ static AVStream *add_video_stream( mlt_consumer this, AVFormatContext *oc, int c
 #else
 		c->pix_fmt = pix_fmt ? avcodec_get_pix_fmt( pix_fmt ) : PIX_FMT_YUV420P;
 #endif
+		
+#if LIBAVCODEC_VERSION_INT > ((52<<16)+(28<<8)+0)
+		switch ( colorspace )
+		{
+		case 170:
+			c->colorspace = AVCOL_SPC_SMPTE170M;
+			break;
+		case 240:
+			c->colorspace = AVCOL_SPC_SMPTE240M;
+			break;
+		case 470:
+			c->colorspace = AVCOL_SPC_BT470BG;
+			break;
+		case 601:
+			c->colorspace = ( 576 % c->height ) ? AVCOL_SPC_SMPTE170M : AVCOL_SPC_BT470BG;
+			break;
+		case 709:
+			c->colorspace = AVCOL_SPC_BT709;
+			break;
+		}
+#endif
 
 		if ( mlt_properties_get( properties, "aspect" ) )
 		{
@@ -772,13 +863,21 @@ static int open_video(AVFormatContext *oc, AVStream *st)
 	}
 
 	// Open the codec safely
-	return codec != NULL && avcodec_open( video_enc, codec ) >= 0;
+	avformat_lock();
+	int result = codec != NULL && avcodec_open( video_enc, codec ) >= 0;
+	avformat_unlock();
+	
+	return result;
 }
 
 void close_video(AVFormatContext *oc, AVStream *st)
 {
 	if ( st && st->codec )
+	{
+		avformat_lock();
 		avcodec_close(st->codec);
+		avformat_unlock();
+	}
 }
 
 static inline long time_difference( struct timeval *time1 )
@@ -827,13 +926,13 @@ static void *consumer_thread( void *arg )
 	int samples = 0;
 
 	// AVFormat audio buffer and frame size
-	int audio_outbuf_size = ( 1024 * 42 );
+	int audio_outbuf_size = AUDIO_BUFFER_SIZE;
 	uint8_t *audio_outbuf = av_malloc( audio_outbuf_size );
 	int audio_input_frame_size = 0;
 
 	// AVFormat video buffer and frame count
 	int frame_count = 0;
-	int video_outbuf_size = ( 1024 * 1024 );
+	int video_outbuf_size = VIDEO_BUFFER_SIZE;
 	uint8_t *video_outbuf = av_malloc( video_outbuf_size );
 
 	// Used for the frame properties
@@ -1128,9 +1227,20 @@ static void *consumer_thread( void *arg )
 
 					// Get the audio samples
 					if ( n > 0 )
+					{
 						sample_fifo_fetch( fifo, audio_buf_1, n );
+					}
+					else if ( audio_codec_id == CODEC_ID_VORBIS && terminated )
+					{
+						// This prevents an infinite loop when some versions of vorbis do not
+						// increment pts when encoding silence.
+						audio_pts = video_pts;
+						break;
+					}
 					else
+					{
 						memset( audio_buf_1, 0, AUDIO_ENCODE_BUFFER_SIZE );
+					}
 					samples = n / channels;
 
 					// For each output stream
@@ -1229,7 +1339,11 @@ static void *consumer_thread( void *arg )
 						if ( pkt.size > 0 )
 						{
 							if ( av_interleaved_write_frame( oc, &pkt ) )
-								mlt_log_error( MLT_CONSUMER_SERVICE( this ), "error writing audio frame %d\n", frames - 1 );
+							{
+								mlt_log_fatal( MLT_CONSUMER_SERVICE( this ), "error writing audio frame\n" );
+								mlt_events_fire( properties, "consumer-fatal-error", NULL );
+								goto on_fatal_error;
+							}
 						}
 
 						mlt_log_debug( MLT_CONSUMER_SERVICE( this ), " frame_size %d\n", codec->frame_size );
@@ -1249,7 +1363,7 @@ static void *consumer_thread( void *arg )
 				// Write video
 				if ( mlt_deque_count( queue ) )
 				{
-					int out_size, ret;
+					int out_size, ret = 0;
 					AVCodecContext *c;
 
 					frame = mlt_deque_pop_front( queue );
@@ -1385,6 +1499,12 @@ static void *consumer_thread( void *arg )
 						}
  					}
  					frame_count++;
+					if ( ret )
+					{
+						mlt_log_fatal( MLT_CONSUMER_SERVICE( this ), "error writing video frame\n" );
+						mlt_events_fire( properties, "consumer-fatal-error", NULL );
+						goto on_fatal_error;
+					}
 					mlt_frame_close( frame );
 				}
 				else
@@ -1448,8 +1568,9 @@ static void *consumer_thread( void *arg )
 			pkt.data = audio_outbuf;
 			if ( av_interleaved_write_frame( oc, &pkt ) != 0 )
 			{
-				mlt_log_error( MLT_CONSUMER_SERVICE( this ), "%s: error writing flushed audio frame\n", __FILE__ );
-				break;
+				mlt_log_fatal( MLT_CONSUMER_SERVICE( this ), "error writing flushed audio frame\n" );
+				mlt_events_fire( properties, "consumer-fatal-error", NULL );
+				goto on_fatal_error;
 			}
 		}
 
@@ -1476,8 +1597,9 @@ static void *consumer_thread( void *arg )
 			// write the compressed frame in the media file
 			if ( av_interleaved_write_frame( oc, &pkt ) != 0 )
 			{
-				mlt_log_error( MLT_CONSUMER_SERVICE(this), "error writing flushed video frame\n" );
-				break;
+				mlt_log_fatal( MLT_CONSUMER_SERVICE(this), "error writing flushed video frame\n" );
+				mlt_events_fire( properties, "consumer-fatal-error", NULL );
+				goto on_fatal_error;
 			}
 			// Dual pass logging
 			if ( mlt_properties_get_data( properties, "_logfile", NULL ) && c->stats_out )
@@ -1485,6 +1607,8 @@ static void *consumer_thread( void *arg )
 		}
 	}
 
+on_fatal_error:
+	
 	// Write the trailer, if any
 	av_write_trailer( oc );
 
