@@ -30,16 +30,16 @@
 #include <framework/mlt_cache.h>
 
 // ffmpeg Header files
-#include <avformat.h>
-#include <opt.h>
+#include <libavformat/avformat.h>
+#include <libavcodec/opt.h>
 #ifdef SWSCALE
-#  include <swscale.h>
+#  include <libswscale/swscale.h>
 #endif
 #if (LIBAVCODEC_VERSION_INT >= ((51<<16)+(71<<8)+0))
 #  include "audioconvert.h"
 #endif
 #ifdef VDPAU
-#include <vdpau.h>
+#include <libavcodec/vdpau.h>
 #endif
 
 // System header files
@@ -95,6 +95,7 @@ struct producer_avformat_s
 	unsigned int invalid_pts_counter;
 	double resample_factor;
 	mlt_cache image_cache;
+	int colorspace;
 #ifdef VDPAU
 	struct
 	{
@@ -262,7 +263,14 @@ static mlt_properties find_default_streams( mlt_properties meta_media, AVFormatC
 					*video_index = i;
 				mlt_properties_set( meta_media, key, "video" );
 				snprintf( key, sizeof(key), "meta.media.%d.stream.frame_rate", i );
+#if LIBAVFORMAT_VERSION_INT >= ((52<<16)+(42<<8)+0)
+				double ffmpeg_fps = av_q2d( context->streams[ i ]->avg_frame_rate );
+				if (ffmpeg_fps == 0) ffmpeg_fps = av_q2d( context->streams[ i ]->r_frame_rate );
+				mlt_properties_set_double( meta_media, key, ffmpeg_fps );
+#else
 				mlt_properties_set_double( meta_media, key, av_q2d( context->streams[ i ]->r_frame_rate ) );
+#endif
+
 #if LIBAVFORMAT_VERSION_INT >= ((52<<16)+(21<<8)+0)
 				snprintf( key, sizeof(key), "meta.media.%d.stream.sample_aspect_ratio", i );
 				mlt_properties_set_double( meta_media, key, av_q2d( context->streams[ i ]->sample_aspect_ratio ) );
@@ -274,6 +282,26 @@ static mlt_properties find_default_streams( mlt_properties meta_media, AVFormatC
 				mlt_properties_set( meta_media, key, avcodec_get_pix_fmt_name( codec_context->pix_fmt ) );
 				snprintf( key, sizeof(key), "meta.media.%d.codec.sample_aspect_ratio", i );
 				mlt_properties_set_double( meta_media, key, av_q2d( codec_context->sample_aspect_ratio ) );
+#if LIBAVCODEC_VERSION_INT > ((52<<16)+(28<<8)+0)
+				snprintf( key, sizeof(key), "meta.media.%d.codec.colorspace", i );
+				switch ( codec_context->colorspace )
+				{
+				case AVCOL_SPC_SMPTE240M:
+					mlt_properties_set_int( meta_media, key, 240 );
+					break;
+				case AVCOL_SPC_BT470BG:
+				case AVCOL_SPC_SMPTE170M:
+					mlt_properties_set_int( meta_media, key, 601 );
+					break;
+				case AVCOL_SPC_BT709:
+					mlt_properties_set_int( meta_media, key, 709 );
+					break;
+				default:
+					// This is a heuristic Charles Poynton suggests in "Digital Video and HDTV"
+					mlt_properties_set_int( meta_media, key, codec_context->width * codec_context->height > 750000 ? 709 : 601 );
+					break;
+				}
+#endif
 				break;
 			case CODEC_TYPE_AUDIO:
 				if ( *audio_index < 0 )
@@ -332,7 +360,7 @@ static int dv_is_wide( AVPacket *pkt )
 	return 0;
 }
 
-static double get_aspect_ratio( AVStream *stream, AVCodecContext *codec_context, AVPacket *pkt )
+static double get_aspect_ratio( mlt_properties properties, AVStream *stream, AVCodecContext *codec_context, AVPacket *pkt )
 {
 	double aspect_ratio = 1.0;
 
@@ -342,15 +370,29 @@ static double get_aspect_ratio( AVStream *stream, AVCodecContext *codec_context,
 		{
 			if ( dv_is_pal( pkt ) )
 			{
-				aspect_ratio = dv_is_wide( pkt )
-					? 64.0/45.0 // 16:9 PAL
-					: 16.0/15.0; // 4:3 PAL
+				if ( dv_is_wide( pkt ) )
+				{
+					mlt_properties_set_int( properties, "meta.media.sample_aspect_num", 64 );
+					mlt_properties_set_int( properties, "meta.media.sample_aspect_den", 45 );
+				}
+				else
+				{
+					mlt_properties_set_int( properties, "meta.media.sample_aspect_num", 16 );
+					mlt_properties_set_int( properties, "meta.media.sample_aspect_den", 15 );
+				}
 			}
 			else
 			{
-				aspect_ratio = dv_is_wide( pkt )
-					? 32.0/27.0 // 16:9 NTSC
-					: 8.0/9.0; // 4:3 NTSC
+				if ( dv_is_wide( pkt ) )
+				{
+					mlt_properties_set_int( properties, "meta.media.sample_aspect_num", 32 );
+					mlt_properties_set_int( properties, "meta.media.sample_aspect_den", 27 );
+				}
+				else
+				{
+					mlt_properties_set_int( properties, "meta.media.sample_aspect_num", 8 );
+					mlt_properties_set_int( properties, "meta.media.sample_aspect_den", 9 );
+				}
 			}
 		}
 		else
@@ -368,13 +410,29 @@ static double get_aspect_ratio( AVStream *stream, AVCodecContext *codec_context,
 			// the rescale normaliser when using equivalent producers and consumers.
 			// = display_aspect / (width * height)
 			if ( ar.num == 10 && ar.den == 11 )
-				aspect_ratio = 8.0/9.0; // 4:3 NTSC
+			{
+				// 4:3 NTSC
+				mlt_properties_set_int( properties, "meta.media.sample_aspect_num", 8 );
+				mlt_properties_set_int( properties, "meta.media.sample_aspect_den", 9 );
+			}
 			else if ( ar.num == 59 && ar.den == 54 )
-				aspect_ratio = 16.0/15.0; // 4:3 PAL
+			{
+				// 4:3 PAL
+				mlt_properties_set_int( properties, "meta.media.sample_aspect_num", 16 );
+				mlt_properties_set_int( properties, "meta.media.sample_aspect_den", 15 );
+			}
 			else if ( ar.num == 40 && ar.den == 33 )
-				aspect_ratio = 32.0/27.0; // 16:9 NTSC
+			{
+				// 16:9 NTSC
+				mlt_properties_set_int( properties, "meta.media.sample_aspect_num", 32 );
+				mlt_properties_set_int( properties, "meta.media.sample_aspect_den", 27 );
+			}
 			else if ( ar.num == 118 && ar.den == 81 )
-				aspect_ratio = 64.0/45.0; // 16:9 PAL
+			{
+				// 16:9 PAL
+				mlt_properties_set_int( properties, "meta.media.sample_aspect_num", 64 );
+				mlt_properties_set_int( properties, "meta.media.sample_aspect_den", 45 );
+			}
 		}
 	}
 	else
@@ -387,10 +445,25 @@ static double get_aspect_ratio( AVStream *stream, AVCodecContext *codec_context,
 			{ 0, 1 };
 #endif
 		if ( codec_sar.num > 0 )
-			aspect_ratio = av_q2d( codec_sar );
+		{
+			mlt_properties_set_int( properties, "meta.media.sample_aspect_num", codec_sar.num );
+			mlt_properties_set_int( properties, "meta.media.sample_aspect_den", codec_sar.den );
+		}
 		else if ( stream_sar.num > 0 )
-			aspect_ratio = av_q2d( stream_sar );
+		{
+			mlt_properties_set_int( properties, "meta.media.sample_aspect_num", stream_sar.num );
+			mlt_properties_set_int( properties, "meta.media.sample_aspect_den", stream_sar.den );
+		}
+		else
+		{
+			mlt_properties_set_int( properties, "meta.media.sample_aspect_num", 1 );
+			mlt_properties_set_int( properties, "meta.media.sample_aspect_den", 1 );
+		}
 	}
+	AVRational ar = { mlt_properties_get_double( properties, "meta.media.sample_aspect_num" ), mlt_properties_get_double( properties, "meta.media.sample_aspect_den" ) };
+	aspect_ratio = av_q2d( ar );
+	mlt_properties_set_double( properties, "aspect_ratio", aspect_ratio );
+
 	return aspect_ratio;
 }
 
@@ -562,23 +635,23 @@ static int producer_open( producer_avformat this, mlt_profile profile, char *fil
 						ret = av_read_frame( context, &pkt );
 						if ( ret >= 0 && pkt.stream_index == video_index && pkt.size > 0 )
 						{
-							mlt_properties_set_double( properties, "aspect_ratio",
-								get_aspect_ratio( context->streams[ video_index ], codec_context, &pkt ) );
+							get_aspect_ratio( properties, context->streams[ video_index ], codec_context, &pkt );
 							break;
 						}
 					}
 				}
 				else
 				{
-					mlt_properties_set_double( properties, "aspect_ratio",
-						get_aspect_ratio( context->streams[ video_index ], codec_context, NULL ) );
+					get_aspect_ratio( properties, context->streams[ video_index ], codec_context, NULL );
 				}
+#ifdef SWSCALE
 				struct SwsContext *context = sws_getContext( codec_context->width, codec_context->height, codec_context->pix_fmt,
 					codec_context->width, codec_context->height, PIX_FMT_YUYV422, SWS_BILINEAR, NULL, NULL, NULL);
 				if ( context )
 					sws_freeContext( context );
 				else
 					error = 1;
+#endif
 			}
 
 			// Read Metadata
@@ -682,9 +755,45 @@ static void get_audio_streams_info( producer_avformat this )
 	this->resample_factor = 1.0;
 }
 
-static inline void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt, mlt_image_format *format, int width, int height )
+static void set_luma_transfer( struct SwsContext *context, int colorspace, int use_full_range )
+{
+#if defined(SWSCALE) && (LIBSWSCALE_VERSION_INT >= ((0<<16)+(7<<8)+2))
+	int *coefficients;
+	int full_range;
+	int brightness, contrast, saturation;
+
+	if ( sws_getColorspaceDetails( context, &coefficients, &full_range, &coefficients, &full_range,
+			&brightness, &contrast, &saturation ) != -1 )
+	{
+		// Don't change these from defaults unless explicitly told to.
+		if ( use_full_range >= 0 )
+			full_range = use_full_range;
+		switch ( colorspace )
+		{
+		case 170:
+		case 470:
+		case 601:
+		case 624:
+			coefficients = sws_getCoefficients( SWS_CS_ITU601 );
+			break;
+		case 240:
+			coefficients = sws_getCoefficients( SWS_CS_SMPTE240M );
+			break;
+		case 709:
+			coefficients = sws_getCoefficients( SWS_CS_ITU709 );
+			break;
+		}
+		sws_setColorspaceDetails( context, coefficients, full_range, coefficients, full_range,
+			brightness, contrast, saturation );
+	}
+#endif
+}
+
+static inline void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
+	mlt_image_format *format, int width, int height, int colorspace )
 {
 #ifdef SWSCALE
+	int full_range = -1;
 	int flags = SWS_BILINEAR | SWS_ACCURATE_RND;
 
 #ifdef USE_MMX
@@ -701,6 +810,7 @@ static inline void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
 			width, height, PIX_FMT_RGBA, flags, NULL, NULL, NULL);
 		AVPicture output;
 		avpicture_fill( &output, buffer, PIX_FMT_RGBA, width, height );
+		set_luma_transfer( context, colorspace, full_range );
 		sws_scale( context, frame->data, frame->linesize, 0, height,
 			output.data, output.linesize);
 		sws_freeContext( context );
@@ -716,6 +826,7 @@ static inline void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
 		output.linesize[0] = width;
 		output.linesize[1] = width >> 1;
 		output.linesize[2] = width >> 1;
+		set_luma_transfer( context, colorspace, full_range );
 		sws_scale( context, frame->data, frame->linesize, 0, height,
 			output.data, output.linesize);
 		sws_freeContext( context );
@@ -726,6 +837,7 @@ static inline void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
 			width, height, PIX_FMT_RGB24, flags | SWS_FULL_CHR_H_INT, NULL, NULL, NULL);
 		AVPicture output;
 		avpicture_fill( &output, buffer, PIX_FMT_RGB24, width, height );
+		set_luma_transfer( context, colorspace, full_range );
 		sws_scale( context, frame->data, frame->linesize, 0, height,
 			output.data, output.linesize);
 		sws_freeContext( context );
@@ -736,6 +848,7 @@ static inline void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
 			width, height, PIX_FMT_RGBA, flags | SWS_FULL_CHR_H_INT, NULL, NULL, NULL);
 		AVPicture output;
 		avpicture_fill( &output, buffer, PIX_FMT_RGBA, width, height );
+		set_luma_transfer( context, colorspace, full_range );
 		sws_scale( context, frame->data, frame->linesize, 0, height,
 			output.data, output.linesize);
 		sws_freeContext( context );
@@ -746,6 +859,7 @@ static inline void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
 			width, height, PIX_FMT_YUYV422, flags | SWS_FULL_CHR_H_INP, NULL, NULL, NULL);
 		AVPicture output;
 		avpicture_fill( &output, buffer, PIX_FMT_YUYV422, width, height );
+		set_luma_transfer( context, colorspace, full_range );
 		sws_scale( context, frame->data, frame->linesize, 0, height,
 			output.data, output.linesize);
 		sws_freeContext( context );
@@ -912,7 +1026,8 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 	int ignore = 0;
 
 	// We may want to use the source fps if available
-	double source_fps = mlt_properties_get_double( properties, "source_fps" );
+	double source_fps = mlt_properties_get_double( properties, "meta.media.frame_rate_num" ) /
+		mlt_properties_get_double( properties, "meta.media.frame_rate_den" );
 	double fps = mlt_producer_get_fps( producer );
 
 	// This is the physical frame position in the source
@@ -1041,11 +1156,13 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 				picture.linesize[0] = codec_context->width;
 				picture.linesize[1] = codec_context->width / 2;
 				picture.linesize[2] = codec_context->width / 2;
-				convert_image( (AVFrame*) &picture, *buffer, PIX_FMT_YUV420P, format, *width, *height );
+				convert_image( (AVFrame*) &picture, *buffer,
+					PIX_FMT_YUV420P, format, *width, *height, this->colorspace );
 			}
 			else
 #endif
-			convert_image( this->av_frame, *buffer, codec_context->pix_fmt, format, *width, *height );
+			convert_image( this->av_frame, *buffer, codec_context->pix_fmt,
+				format, *width, *height, this->colorspace );
 		}
 		else
 			mlt_frame_get_image( frame, buffer, format, width, height, writable );
@@ -1189,11 +1306,6 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 					}
 				}
 				mlt_log_debug( MLT_PRODUCER_SERVICE(producer), " got_pic %d key %d\n", got_picture, pkt.flags & PKT_FLAG_KEY );
-				av_free_packet( &pkt );
-			}
-			else if ( ret >= 0 )
-			{
-				av_free_packet( &pkt );
 			}
 
 			// Now handle the picture if we have one
@@ -1227,7 +1339,8 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 							VdpStatus status = vdp_surface_get_bits( render->surface, dest_format, planes, pitches );
 							if ( status == VDP_STATUS_OK )
 							{
-								convert_image( (AVFrame*) &picture, *buffer, PIX_FMT_YUV420P, format, *width, *height );
+								convert_image( (AVFrame*) &picture, *buffer, PIX_FMT_YUV420P,
+									format, *width, *height, this->colorspace );
 							}
 							else
 							{
@@ -1243,7 +1356,8 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 					}
 					else
 #endif
-					convert_image( this->av_frame, *buffer, codec_context->pix_fmt, format, *width, *height );
+					convert_image( this->av_frame, *buffer, codec_context->pix_fmt,
+						format, *width, *height, this->colorspace );
 					this->top_field_first |= this->av_frame->top_field_first;
 					this->current_position = int_position;
 					this->got_picture = 1;
@@ -1253,6 +1367,8 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 					got_picture = 0;
 				}
 			}
+			if ( ret >= 0 )
+				av_free_packet( &pkt );
 		}
 	}
 
@@ -1282,6 +1398,10 @@ exit_get_image:
 	else
 		mlt_properties_set_int( frame_properties, "top_field_first", this->top_field_first );
 
+	// Set immutable properties of the selected track's (or overridden) source attributes.
+	mlt_properties_set_int( properties, "meta.media.top_field_first", this->top_field_first );
+	mlt_properties_set_int( properties, "meta.media.progressive", mlt_properties_get_int( frame_properties, "progressive" ) );
+
 	return !this->got_picture;
 }
 
@@ -1375,7 +1495,7 @@ static int video_codec_init( producer_avformat this, int index, mlt_properties p
 		mlt_properties_set_int( properties, "height", this->video_codec->height );
 		// For DV, we'll just use the saved aspect ratio
 		if ( codec_context->codec_id != CODEC_ID_DVVIDEO )
-			mlt_properties_set_double( properties, "aspect_ratio", get_aspect_ratio( stream, this->video_codec, NULL ) );
+			get_aspect_ratio( properties, stream, this->video_codec, NULL );
 
 		// Determine the fps first from the codec
 		double source_fps = (double) this->video_codec->time_base.den /
@@ -1384,21 +1504,82 @@ static int video_codec_init( producer_avformat this, int index, mlt_properties p
 		if ( mlt_properties_get( properties, "force_fps" ) )
 		{
 			source_fps = mlt_properties_get_double( properties, "force_fps" );
-			stream->time_base = av_d2q( source_fps, 255 );
+			stream->time_base = av_d2q( source_fps, 1024 );
+			mlt_properties_set_int( properties, "meta.media.frame_rate_num", stream->time_base.num );
+			mlt_properties_set_int( properties, "meta.media.frame_rate_den", stream->time_base.den );
 		}
 		else
 		{
 			// If the muxer reports a frame rate different than the codec
+#if LIBAVFORMAT_VERSION_INT >= ((52<<16)+(42<<8)+0)
+			double muxer_fps = av_q2d( stream->avg_frame_rate );
+			if ( muxer_fps == 0 ) muxer_fps = av_q2d( stream->r_frame_rate );
+#else
 			double muxer_fps = av_q2d( stream->r_frame_rate );
+#endif
 			// Choose the lesser - the wrong tends to be off by some multiple of 10
 			source_fps = FFMIN( source_fps, muxer_fps );
+			if ( source_fps >= 1.0 && ( source_fps < muxer_fps || isnan( muxer_fps ) ) )
+			{
+				mlt_properties_set_int( properties, "meta.media.frame_rate_num", this->video_codec->time_base.den );
+				mlt_properties_set_int( properties, "meta.media.frame_rate_den", this->video_codec->time_base.num == 0 ? 1 : this->video_codec->time_base.num );
+			}
+			else if ( muxer_fps > 0 )
+			{
+				AVRational frame_rate = stream->r_frame_rate;
+				// With my samples when r_frame_rate != 1000 but avg_frame_rate is valid,
+				// avg_frame_rate gives some approximate value that does not well match the media.
+				// Also, on my sample where r_frame_rate = 1000, using avg_frame_rate directly
+				// results in some very choppy output, but some value slightly different works
+				// great.
+#if LIBAVFORMAT_VERSION_INT >= ((52<<16)+(42<<8)+0)
+				if ( av_q2d( stream->r_frame_rate ) >= 1000 && av_q2d( stream->avg_frame_rate ) > 0 )
+					frame_rate = av_d2q( av_q2d( stream->avg_frame_rate ), 1024 );
+#endif
+				mlt_properties_set_int( properties, "meta.media.frame_rate_num", frame_rate.num );
+				mlt_properties_set_int( properties, "meta.media.frame_rate_den", frame_rate.den );
+			}
+			else
+			{
+				source_fps = mlt_producer_get_fps( this->parent );
+				AVRational frame_rate = av_d2q( source_fps, 255 );
+				mlt_properties_set_int( properties, "meta.media.frame_rate_num", frame_rate.num );
+				mlt_properties_set_int( properties, "meta.media.frame_rate_den", frame_rate.den );
+			}
 		}
 
-		// We'll use fps if it's available
+		// source_fps is deprecated in favor of meta.media.frame_rate_num and .frame_rate_den
 		if ( source_fps > 0 )
 			mlt_properties_set_double( properties, "source_fps", source_fps );
 		else
 			mlt_properties_set_double( properties, "source_fps", mlt_producer_get_fps( this->parent ) );
+
+		// Set the YUV colorspace from override or detect
+		this->colorspace = mlt_properties_get_int( properties, "force_colorspace" );
+#if LIBAVCODEC_VERSION_INT > ((52<<16)+(28<<8)+0)		
+		if ( ! this->colorspace )
+		{
+			switch ( this->video_codec->colorspace )
+			{
+			case AVCOL_SPC_SMPTE240M:
+				this->colorspace = 240;
+				break;
+			case AVCOL_SPC_BT470BG:
+			case AVCOL_SPC_SMPTE170M:
+				this->colorspace = 601;
+				break;
+			case AVCOL_SPC_BT709:
+				this->colorspace = 709;
+				break;
+			default:
+				// This is a heuristic Charles Poynton suggests in "Digital Video and HDTV"
+				this->colorspace = this->video_codec->width * this->video_codec->height > 750000 ? 709 : 601;
+				break;
+			}
+		}
+#endif
+		// Let apps get chosen colorspace
+		mlt_properties_set_int( properties, "meta.media.colorspace", this->colorspace );
 	}
 	return this->video_codec && this->video_index > -1;
 }
@@ -1486,13 +1667,20 @@ static void producer_set_up_video( producer_avformat this, mlt_frame frame )
 		// Set the width and height
 		mlt_properties_set_int( frame_properties, "width", this->video_codec->width );
 		mlt_properties_set_int( frame_properties, "height", this->video_codec->height );
+		// real_width and real_height are deprecated in favor of meta.media.width and .height
+		mlt_properties_set_int( properties, "meta.media.width", this->video_codec->width );
+		mlt_properties_set_int( properties, "meta.media.height", this->video_codec->height );
 		mlt_properties_set_int( frame_properties, "real_width", this->video_codec->width );
 		mlt_properties_set_int( frame_properties, "real_height", this->video_codec->height );
 		mlt_properties_set_double( frame_properties, "aspect_ratio", aspect_ratio );
+		mlt_properties_set_int( frame_properties, "colorspace", this->colorspace );
 
 		// Workaround 1088 encodings missing cropping info.
 		if ( this->video_codec->height == 1088 && mlt_profile_dar( mlt_service_profile( MLT_PRODUCER_SERVICE( producer ) ) ) == 16.0/9.0 )
+		{
+			mlt_properties_set_int( properties, "meta.media.height", 1080 );
 			mlt_properties_set_int( frame_properties, "real_height", 1080 );
+		}
 
 		// Add our image operation
 		mlt_frame_push_service( frame, this );
