@@ -22,6 +22,8 @@
 #include <string.h>
 
 extern mlt_consumer consumer_null_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
+extern mlt_filter filter_audioconvert_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
+extern mlt_filter filter_audiowave_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_brightness_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_channelcopy_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_crop_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
@@ -29,10 +31,12 @@ extern mlt_filter filter_data_feed_init( mlt_profile profile, mlt_service_type t
 extern mlt_filter filter_data_show_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_gamma_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_greyscale_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
+extern mlt_filter filter_imageconvert_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_luma_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_mirror_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_mono_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_obscure_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
+extern mlt_filter filter_panner_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_region_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_rescale_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_resize_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
@@ -52,23 +56,29 @@ extern mlt_transition transition_mix_init( mlt_profile profile, mlt_service_type
 MLT_REPOSITORY
 {
 	MLT_REGISTER( consumer_type, "null", consumer_null_init );
+	MLT_REGISTER( filter_type, "audioconvert", filter_audioconvert_init );
+	MLT_REGISTER( filter_type, "audiowave", filter_audiowave_init );
 	MLT_REGISTER( filter_type, "brightness", filter_brightness_init );
 	MLT_REGISTER( filter_type, "channelcopy", filter_channelcopy_init );
-	MLT_REGISTER( filter_type, "crop", filter_crop_init );
+    MLT_REGISTER( filter_type, "channelswap", filter_channelcopy_init );
+    MLT_REGISTER( filter_type, "crop", filter_crop_init );
 	MLT_REGISTER( filter_type, "data_feed", filter_data_feed_init );
 	MLT_REGISTER( filter_type, "data_show", filter_data_show_init );
 	MLT_REGISTER( filter_type, "gamma", filter_gamma_init );
 	MLT_REGISTER( filter_type, "greyscale", filter_greyscale_init );
 	MLT_REGISTER( filter_type, "grayscale", filter_greyscale_init );
+	MLT_REGISTER( filter_type, "imageconvert", filter_imageconvert_init );
 	MLT_REGISTER( filter_type, "luma", filter_luma_init );
 	MLT_REGISTER( filter_type, "mirror", filter_mirror_init );
 	MLT_REGISTER( filter_type, "mono", filter_mono_init );
 	MLT_REGISTER( filter_type, "obscure", filter_obscure_init );
+	MLT_REGISTER( filter_type, "panner", filter_panner_init );
 	MLT_REGISTER( filter_type, "region", filter_region_init );
 	MLT_REGISTER( filter_type, "rescale", filter_rescale_init );
 	MLT_REGISTER( filter_type, "resize", filter_resize_init );
 	MLT_REGISTER( filter_type, "transition", filter_transition_init );
 	MLT_REGISTER( filter_type, "watermark", filter_watermark_init );
+	MLT_REGISTER( producer_type, "abnormal", producer_loader_init );
 	MLT_REGISTER( producer_type, "color", producer_colour_init );
 	MLT_REGISTER( producer_type, "colour", producer_colour_init );
 	MLT_REGISTER( producer_type, "consumer", producer_consumer_init );
